from enum import Enum

from pydantic import BaseModel


class ScenarioType(str, Enum):
    CHAT_APP = "CHAT_APP"


class ScenarioInfo(BaseModel):
    key: str
    name: str
    code: str
    version: str
    type: ScenarioType
    payload: dict



from abc import abstractmethod, ABC
from typing import Dict, Any, Optional


class StoreProvider(ABC):
    NAME = ""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @abstractmethod
    async def init(self, *args, **kwargs):
        pass

    @abstractmethod
    async def connect(self, *args, **kwargs) -> bool:
        pass

    @abstractmethod
    async def save(self, key: str, value: Dict[str, Any], raw_table: str) -> bool:
        pass

    @abstractmethod
    async def save_or_update(
        self, key: str, value: Dict[str, Any], raw_table: str
    ) -> bool:
        pass

    @abstractmethod
    async def get(self, key: str, raw_table: str) -> Optional[Dict[str, Any]]:
        pass

    @abstractmethod
    async def remove(self, key: str, raw_table: str) -> bool:
        pass



from pydantic import BaseModel

from salute.common.models.message.payload import Payload


class Suggestion(BaseModel):
    buttons: list[dict] = []


class ResponsePayload(Payload):
    pronounceText: str = ""
    items: list = []
    suggestions: Suggestion = Suggestion()



from typing import Annotated

from pydantic import BaseModel, StringConstraints

LowerCaseText = Annotated[str | None, StringConstraints(to_lower=True)]


class Text(BaseModel):
    original_text: LowerCaseText = ""
    normalized_text: LowerCaseText = ""
    human_normalized_text: LowerCaseText = ""
    asr_normalized_message: LowerCaseText = ""
    human_normalized_text_with_anaphora: LowerCaseText = ""



from enum import Enum


class Name(str, Enum):
    MESSAGE_TO_SKILL = "MESSAGE_TO_SKILL"
    CLOSE_APP = "CLOSE_APP"
    RUN_APP = "RUN_APP"
    ERROR = "ERROR"
    NOTHING_FOUND = "NOTHING_FOUND"
    ANSWER_TO_USER = "ANSWER_TO_USER"



from sanic import Sanic
from sanic.logging.loggers import logger
from sanic.request import Request
from sanic.response import json

from salute.common.models.message.name import Name
from salute.common.models.request_message import RequestMessage
from salute.common.models.response_message import ResponseMessage
from salute.common.models.scenario.scenario import Scenario
from salute.common.store.store import StoreProvider


class App(Sanic):
    def __init__(self, name: str, *args, **kwargs) -> None:
        super().__init__(name, *args, **kwargs)

        self._setup_scenario_router()

    def _setup_scenario_router(self) -> None:
        @self.post(self.config.APP_CONNECTOR_ROUTE)
        async def app_connector(request: Request) -> json:
            logger.info("Get request message")
            logger.error(request.json)
            data = RequestMessage(**request.json)
            logger.info("Data prepared")

            logger.info("Check root intent")
            project = data.payload.app_info.projectId
            logger.info(f"Project ID: {project}")

            scenario: Scenario | None = self.get_scenario(project)

            if scenario:
                logger.info(
                    f"Scenario {scenario.info.name} {project} is found"
                )
                response = await scenario.app.process(data)
                return json(response.model_dump(mode="json"), 200)

            logger.info(f"Nothing found for {project}")
            response = ResponseMessage(**data.model_dump(mode="json"))
            response.messageName = Name.NOTHING_FOUND

            return json(response.model_dump(mode="json"), 200)

    def get_storage(self) -> StoreProvider:
        return self.config.STORAGE

    def get_scenario(self, project_id: str) -> Scenario | None:
        return self.config["SCENARIOS"].get(project_id, None)



import importlib
from pathlib import Path
from sanic.logging.loggers import logger


SPECIAL_DIRS = ["__pycache__", ".git"]


class ScenarioConnector:
    SCENARIO_SRC = "scenarios"
    SCENARIO_CONNECT_METHOD = "scenario_connector"

    def __init__(self):
        p = Path().resolve()
        self.scenarios_dirs = p / self.SCENARIO_SRC

    async def connect(self) -> None:
        connected_count = 0

        for full_path in self.scenarios_dirs.iterdir():
            directory = full_path.parts[len(full_path.parts) - 1]

            if not full_path.is_dir() or directory in SPECIAL_DIRS:
                continue

            logger.info(f"Connecting to scenario {directory}")

            try:
                mod = importlib.import_module(f"{self.SCENARIO_SRC}.{directory}")
                process = getattr(mod, self.SCENARIO_CONNECT_METHOD)
                await process(full_path)
                connected_count += 1
                logger.info(f"Scenario {directory} is connected")
            except Exception as e:
                logger.error(f"Failed to connect to scenario {directory}: {e}")

        logger.info(f"Total {connected_count} scenarios connected")



from pathlib import Path
from typing import Callable, Coroutine, Awaitable

from decouple import config
from sanic import Sanic

from salute.common.utils.load_data import load_data
from salute.common.models.request_message import RequestMessage
from salute.common.models.response_message import ResponseMessage
from salute.common.models.scenario.utils import ScenarioInfo
from salute.common.store.store import StoreProvider


class SessionStore:
    get: Callable
    set: Callable
    drop: Callable

    def __init__(self, get, set, drop) -> None:
        self.get = get
        self.set = set
        self.drop = drop


class ScenarioApp:
    def __init__(self, path: Path, *args, **kwargs) -> None:
        self.static_dir = Path(path) / "static"
        self.info: ScenarioInfo = self._set_scenario_config()
        self.storage: StoreProvider = Sanic.get_app(config("NAME")).config.STORAGE
        self.session_store: SessionStore = SessionStore(
            self.get_session,
            self.set_session,
            self.drop_session,
        )

    def _set_scenario_config(self) -> ScenarioInfo:
        data = load_data(self.static_dir / "intent.json")
        return ScenarioInfo(**data)

    async def set_storage(self) -> None:
        await self.storage.init(self.info.key)

    async def get_session(self, key: str) -> dict:
        return await self.storage.get(key, self.info.key)

    async def set_session(self, key: str, data: dict) -> bool:
        return await self.storage.save_or_update(key, data, self.info.key)

    async def drop_session(self, key: str) -> bool:
        return await self.storage.remove(key, self.info.key)

    async def process(self, scenario_request: RequestMessage) -> ResponseMessage:
        pass



from pydantic import BaseModel


class AppInfo(BaseModel):
    projectId: str
    applicationId: str
    appversionId: str
    systemName: str
    projectName: str
    frontendType: str
    ageLimit: int
    affiliationType: str



from salute.common.models.message.message import Message
from salute.common.models.message.request_payload import RequestPayload


class RequestMessage(Message):
    payload: RequestPayload

    def get_intent(self) -> str:
        if not self.payload.intent:
            return ""
        return self.payload.intent.lower()

    def get_user_id(self) -> str:
        return self.uuid.userId



from pydantic import BaseModel


class Device(BaseModel):
    additionalInfo: dict = {}
    capabilities: dict = {}
    deviceId: str = ""
    deviceManufacturer: str = ""
    deviceModel: str = ""
    features: dict = {}
    platformType: str = ""
    platformVersion: str = ""
    surface: str = ""
    surfaceVersion: str = ""



from salute.common.models.message.app_info import *
from salute.common.models.message.device import *
from salute.common.models.message.message import *
from salute.common.models.message.name import *
from salute.common.models.message.payload import *
from salute.common.models.message.request_payload import *
from salute.common.models.message.response_payload import *
from salute.common.models.message.text import *
from salute.common.models.message.user import *



from pydantic import BaseModel
from salute.common.models.scenario.scenario_app import ScenarioApp, SessionStore
from salute.common.models.scenario.utils import ScenarioInfo


class Scenario(BaseModel):
    app: ScenarioApp
    info: ScenarioInfo
    session_store: SessionStore

    class Config:
        arbitrary_types_allowed = True



from decouple import config
from sanic import Config

from salute.common.models.scenario.scenario import Scenario
from salute.common.store.store import StoreProvider


class AppConfig(Config):
    STAND: str = config("STAND", default=False, cast=str)
    NAME: str = config("NAME", default=None, cast=str)
    VERSION: str = config("VERSION", default="0.0.1", cast=str)
    STORAGE: StoreProvider | None = None
    SCENARIOS: dict[str, Scenario] = {}
    APP_CONNECTOR_ROUTE: str = config(
        "APP_CONNECTOR_ROUTE", default="/app-connector", cast=str
    )


app_config = AppConfig()



from salute.common.models.message.app_info import AppInfo
from salute.common.models.message.payload import Payload
from salute.common.models.message.text import Text


class RequestPayload(Payload):
    app_info: AppInfo
    message: Text



from pydantic import BaseModel

from salute.common.models.message.payload import Payload
from salute.common.models.message.user import User
from salute.common.models.message.name import Name


class Message(BaseModel):
    sessionId: str
    messageId: int
    messageName: Name
    payload: Payload
    uuid: User



# from string import Template
# from typing import Dict, Optional
#
# from salute.common.scenario import ScenarioIntent
#
#
# class ScenarioPhrase:
#     value: Template | str = ""
#
#     def get_text(self, **kwargs) -> str:
#         return self.value.substitute(**kwargs)
#
#
# class ScenarioPayload:
#     payload: dict = {}
#
#
# class ScenarioConnector:
#
#     def __init__(self, phrase: type(ScenarioPhrase), payload: ScenarioPayload):
#         self.intent: Optional[ScenarioIntent] = None
#         self.phrases: Dict[str, phrase] = {}
#         self.payload: Dict[str, payload] = {}
#
#     async def connect(self):
#         print(self.phrases)
#         print(type(self.phrases))
#
#
# c = ScenarioConnector(ScenarioPhrase, ScenarioPayload())



def get_bubble(text: str) -> dict:
    return {"bubble": {"text": text}}


def get_card_list(content_url: str, content_hash: str) -> dict:
    return {
        "card": {
            "type": "list_card",
            "cells": [
                {
                    "type": "image_cell_view",
                    "content": {
                        "url": content_url,
                        "hash": content_hash,
                    },
                }
            ],
        }
    }


def get_suggestion(choices: list[str]) -> list[dict]:
    result = []
    for choice in choices:
        result.append(
            {
                "title": choice,
                "actions": [
                    {
                        "type": "text",
                        "text": choice,
                    }
                ],
            }
        )

    return result
